<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
/*
 * Email proceessing for block wcln_progress.
 *
 * @package   block_wcln_progress
 * @copyright 2020 Tim Hunt
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once('../../config.php');
require_once('../../enrol/externallib.php');
require_once('lib.php');
require_once('email_form.php');
require_once('../../lib/weblib.php');

require_login();
$courseid = required_param('courseid', PARAM_INT);
$groupsused = required_param('groups', PARAM_INT);
$type = optional_param('type', '', PARAM_ALPHA);
$typeid = optional_param('typeid', 0, PARAM_INT);
$sigid = optional_param('sigid', 0, PARAM_INT);
$messageidresend = optional_param('fmid', 0, PARAM_INT);

if (!$course = $DB->get_record('course', array('id' => $courseid))) {
    print_error('no_course', 'block_wcln_progress', '', $courseid);
}

if (!empty($type) and !in_array($type, array('log', 'drafts'))) {
    print_error('no_type', 'block_wcln_progress', '', $type);
}

if (!empty($type) and empty($typeid)) {
    $string = new stdclass;
    $string->tpe = $type;
    $string->id = $typeid;

    print_error('no_typeid', 'block_wcln_progress', '', $string);
}

$context = context_course::instance($courseid);
$haspermission = (
        has_capability('block/wcln_progress:cansend', $context) or
        !empty($config['allowstudents'])
        );

if (!$haspermission) {
    print_error('no_permission', 'block_wcln_progress');
}

$altparams = array('courseid' => $course->id, 'valid' => 1);
$alternates = $DB->get_records_menu('block_wcln_alternate', $altparams, '', 'id, address');

$blockname = wcln_progress::_s('pluginname');
$header = wcln_progress::_s('email');

$PAGE->set_context($context);
$PAGE->set_course($course);
$PAGE->navbar->add($blockname);
$PAGE->navbar->add($header);
$PAGE->set_title($blockname . ': ' . $header);
$PAGE->set_heading($blockname . ': ' . $header);
$PAGE->set_url('/blocks/wcln_progress/email.php', array('courseid' => $courseid));
$PAGE->set_pagetype(wcln_progress::PAGE_TYPE);
$PAGE->set_pagelayout('standard');

$courseroles = get_roles_used_in_context($context);
$plugin = 'block_wcln_progress';
$allgroups = groups_get_all_groups($courseid);
$mastercap = true;
$groups = $allgroups;

$globalaccess = empty($allgroups);

// Fill the course users by.
$users = array();
$userstoroles = array();
$userstogroups = array();

$everyone = wcln_progress::get_non_suspended_users($context, $courseid);

if (empty($config['messenger'])) {
    $msg = '0';
} else {
    $msg = $config['messenger'];
}
foreach ($everyone as $userid => $user) {
    if (!$user->suspended) {
        $users[$userid] = $user;
    }
}
if (empty($users)) {
    print_error('no_usergroups', 'block_wcln_progress');
}
if (!empty($type)) {
    $email = $DB->get_record('block_wcln_progress_' . $type, array('id' => $typeid));
    if ($messageidresend == 1) {
        list($email->mailto, $email->additional_emails) = wcln_progress::clean($email->failuserids);
    }
} else {
    $email = new stdClass;
    $email->id = null;
}
$email->messageformat = editors_get_preferred_format();

// Some setters for the form.
$email->type = $type;
$email->typeid = $typeid;

$editoroptions = array(
    'trusttext' => false,
    'subdirs' => 1,
    'maxfiles' => EDITOR_UNLIMITED_FILES,
    'accepted_types' => '*',
    'context' => $context
);

$email = file_prepare_standard_editor(
    $email, 'message', $editoroptions, $context, 'block_wcln_progress', $type, $email->id
);

$selected = array();
if (!empty($email->mailto)) {
    foreach (explode(',', $email->mailto) as $id) {
        $selected[$id] = (object) array('id' => $id, 'firstname' => null, 'lastname' => null, 'email' => $id,
            'mailformat' => '1', 'suspended' => '0', 'maildisplay' => '2', 'status' => '0');
        if (isset($selected[$id]) && isset($users[$id])) {
            if (is_numeric($selected[$id]->id)) {
                $selected[$id] = $users[$id];
            }
        }
        unset($users[$id]);
    }
}

$form = new email_form(null, array(
    'editor_options' => $editoroptions,
    'selected' => $selected,
    'users' => $users,
    'groups' => $groups,
    'userstoroles' => $userstoroles,
    'userstogroups' => $userstogroups,
    'alternates' => $alternates,
    'groupsused' => $groupsused
));

$warnings = array();
if ($form->is_cancelled()) {
    redirect(new moodle_url('/course/view.php?id=' . $courseid));
} else if ($data = $form->get_data()) {
    if (empty($data->subject)) {
        $warnings[] = get_string('no_subject', 'block_wcln_progress');
    }
    if (empty($warnings)) {
        $data->time = time();
        $data->format = $data->message_editor['format'];
        $data->message = $data->message_editor['text'];
        $data->messageWithSigAndAttach = "";
        if (isset($data->send)) {
            $datanew = new stdclass;
            $datanew->courseid = $data->courseid;
            $datanew->userid = $data->userid;
            $string = '';
            $string2 = '';
            $lmail = strlen($data->userid);
            if ($lmail > 2) {
                $string = implode('|', $data->mailto);
            }
            if ($data->mailtogroup <> 0 ) {
                $lgroup = count($data->mailtogroup);
            } else {
                $lgroup = 0;
            }
            if ($lgroup >= 2 ) {
                $string2 = implode ('|', $data->mailtogroup);
            }
            $string = $string.$string2;
            $datanew->mailto = $string;
            $datanew->subject = $data->subject;
            $datanew->message = $data->message;
            $datanew->time = time();
            $data->id = $DB->insert_record('block_wcln_progress_log', $datanew);
            $table = 'log';
        } else {
            $data = file_postupdate_standard_editor(
            $data, 'message', $editoroptions, $context, 'block_wcln_progress', $table, $data->id
            );
        }
        if (!empty($data->subject)) {
            $subject = $data->subject;
        } else {
            $subject = "[{$course->$prepender}] $data->subject";
        }
        // An instance id is needed before storing the file repository.
        if (isset($data->send)) {
            $data->messagewithsigandattach = $data->message;
            // Prepare html content of message.
            $data->message = file_rewrite_pluginfile_urls($data->message_editor['text'], 'pluginfile.php',
                    $context->id, 'block_wcln_progress', $table, $data->id, $editoroptions);
            if (!empty($data->alternateid)) {
                $user = clone($USER);
                $user->email = $alternates[$data->alternateid];
            } else {
                $user = $USER;
            }
            $data->failuserids = array();
            $messagetext = format_text_email($data->messagewithsigandattach, $data->format);
            // HTML.
            $messagehtml = format_text($data->messagewithsigandattach, $data->format);
            $messagetextbase = '';
            $messagehtmlbase = '';
            $from = $DB->get_record('user', array('id' => $USER->id));
            $users = array();
            $cnt = 0;
            if ($data->mailto == 0) {
                $lmail = 0;
            } else {
                $lmail = count($data->mailto);
            }
            if ($lmail >= 1) {
                foreach ($data->mailto as $userlist) {
                    $users[] = $userlist;
                    $cnt = $cnt + 1;
                }
            }
            if ($data->mailtogroup == 0) {
                $cntgroup = 0;
            } else {
                $cntgroup = count($data->mailtogroup);
            }
            if ($cntgroup >= 1) {
                if ($data->mailtogroup > 0) {
                    foreach ($data->mailtogroup as $groupid) {
                        if ($groupid != 0) {
                            $resultgroupmembers = $DB->get_records_sql("
                                    select userid from {groups_members} where groupid =". $groupid);
                            foreach ($resultgroupmembers as $groupuser) {
                                // Check to see if the user is already in the list.
                                $found = 0;
                                foreach ($users as $userid) {
                                    if ($userid == $groupuser) {
                                        $found = 1;
                                    }
                                }
                                if ($found == 0 ) {
                                    $uid = $groupuser->userid;
                                    $users[] = $uid;
                                    $cnt = $cnt + 1;
                                }
                            }
                        }
                    }
                }
            }
            foreach ($users as $userid) {
                if ($data->progress) {
                    $params23 = array($userid);
                    $resultstudent = $DB->get_records_sql("select lastname, firstname from {user} u where u.id = ?", $params23);
                    foreach ($resultstudent as $student) {
                        $nm = $student->firstname.' '.$student->lastname;
                    }
                    $base = 'Student: '.$nm;
                    $params = array($courseid, $userid);
                    $resultlast = $DB->get_records_sql("select * from {user_lastaccess} where courseid=? and
                                                        userid=? order by timeaccess", $params);
                    $ltime = "N/A";
                    foreach ($resultlast as $last) {
                        $lt = $last->timeaccess;
                        $dif = time() - $lt;
                        $d = floor(($dif % 2592000) / 86400);
                        $ltime = $d.' '.get_string('day', $plugin);
                    }
                    $params21 = array($courseid);
                    $resultcontext = $DB->get_records_sql("select id from {context} where contextlevel <= 50 and
                                                           instanceid = ? order by contextlevel desc", $params21);
                    foreach ($resultcontext as $ctx) {
                        $cid = $ctx->id;
                    }
                    $params22 = array($cid);
                    $tl = 'N/A';
                    $tcnt = 0;
                    if (!empty($data->teachers)) {
                        $resultteacher = $DB->get_records_sql("select lastname,firstname, u.id from {role_assignments} r,
                                                              {user} u where u.id = r.userid and roleid = 3  and contextid = ?",
                                                              $params22);
                        foreach ($resultteacher as $tlist) {
                            $teacherid = $tlist->id;
                            if (in_array($teacherid, $data->teachers)) {
                                $tcnt = $tcnt + 1;
                                $tn = $tlist->firstname . ' '. $tlist->lastname;
                                if ($tcnt == 1) {
                                    $tl = $tn;
                                } else {
                                    $tl = $tl.'<br>'.$tn;
                                }
                            }
                        }
                    }
                    $resultcourse = $DB->get_records_sql("select * from {course} where id=?", $params21);
                    foreach ($resultcourse as $crs) {
                        $cn = $crs->fullname;
                    }
                    $base = $base .'<br> Course: <b>'.$cn.'</b>';
                    $rdate = date("Y-m-d");
                    if ($data->cgrade) {
                        $gtot = 0;
                        $resultcitem = $DB->get_records_sql("select * from {grade_items}
                                                             where courseid =? and itemtype = 'course'", $params21);
                        foreach ($resultcitem as $citem) {
                            $mdlid = $citem->id;
                            $maxgrade = $citem->grademax;
                            $params221 = array($courseid, $userid, $mdlid);
                            $resultfinal = $DB->get_records_sql("select finalgrade from {grade_grades} gg,{grade_items} gi
                                                                 where gi.id = gg.itemid and courseid=? and userid=?
                                                                 and gi.id=?  and gg.excluded = 0 ", $params221);
                            foreach ($resultfinal as $cfinal) {
                                $gtot = $cfinal->finalgrade;
                                if ($maxgrade > 100) {
                                    $gtot = $gtot / $maxgrade;
                                    $gtot = $gtot * 100;
                                }
                                $gtot = round($gtot, 1);
                            }
                            $base = $base .'<br>'.get_string('cgrade', $plugin).' <b>'.$gtot .' %</b>';
                        }
                    }
                    if ($data->pcomp) {
                        $rcomp = $DB->count_records_sql("select COUNT(*)
                                      FROM {course_completion_criteria} where course =?", $params21);
                        if ($rcomp >= 1) {
                            $params223 = array($courseid, $userid);
                            $resultcomp = $DB->count_records_sql("select COUNT(*)
                                               FROM {course_completion_crit_compl} where course =?  and userid=?", $params223);
                            $rval = ($resultcomp / $rcomp) * 100;
                            $rval = round($rval, 1);
                            $base = $base .'<br> '.get_string('percentcomp', $plugin).' <b>'.$rval .' %</b>';
                        }
                    }
                    $base = $base .'<br>'.get_string('lasttime', $plugin).' <b>'.$ltime.'</b>';
                    $base = $base .'<br> Teacher(s): '.$tl;
                    $base = $base .'<br> Report Date: '.$rdate;
                    $base = $base .'<br><table border="1"><th><b>Activity Name</b></th><th><b>Category </b></th>
                                     <th><b>Grade</b></th><th><b>Date Submitted</b></th></tr>';
                    $params2 = array($courseid);
                    $result = $DB->get_records_sql("select * from {course_sections} where course =?", $params2);
                    $ln = '';
                    foreach ($result as $seq) {
                        $seqarr = $seq->sequence;
                        $wordchunks = explode(",", $seqarr);
                        for ($i = 0; $i < count($wordchunks); $i++) {
                            $sec = $wordchunks[$i];
                            $params33 = array($sec);
                            $resultmod = $DB->get_records_sql("select * from
                                             {course_modules} where id = ?", $params33);
                            foreach ($resultmod as $mlist) {
                                $mtype = $mlist->module;
                                $modid = $mlist->instance;
                                $params44 = array($courseid, $modid);
                                $result5 = $DB->get_records_sql("select * from {grade_items}
                                                where courseid = ? and hidden = 0 and iteminstance=?", $params44);
                                foreach ($result5 as $item) {
                                    $catid = $item->categoryid;
                                    if (!is_null($catid)) {
                                        $params2 = array($catid);
                                        $resultcat = $DB->get_records_sql("select * from {grade_categories}
                                                     where  id=? ", $params2);
                                        foreach ($resultcat as $cats) {
                                            $cn = trim($cats->fullname);
                                        }
                                        $lc = strlen($cn);
                                        if ($lc > 3) {
                                            $act = $item->itemname;
                                            $mdlid = $item->id;
                                            $mname = $item->itemmodule;
                                            $mgrade = $item->grademax;
                                            $mgrade = round($mgrade, 2);
                                            $ln = $ln.'<tr><td>'.$act.'</td><td>'.$cn.'</td>';
                                            $params3 = array($courseid, $userid, $mdlid);
                                            $resultgrade = $DB->get_records_sql("
                                            select finalgrade, g.excluded, g.timemodified from {grade_grades} g,{grade_items} gi
                                            where gi.id = g.itemid and courseid=?
                                            and gi.hidden = 0 and userid=? and gi.id=? ", $params3);
                                            $cnt = '0';
                                            $grade = 'N/A';
                                            $dmark = 'N/A';
                                            foreach ($resultgrade as $gid) {
                                                $cnt = '1';
                                                $sgrade = $gid->finalgrade;
                                                $exclude = $gid->excluded;
                                                if ($exclude > 0) {
                                                    $grade = "OMIT";
                                                    $dmark = "&nbsp;&nbsp;&nbsp;";
                                                } else {
                                                    if (is_null($sgrade)) {
                                                        $dmark = "N/A";
                                                        $grade = "N/A";
                                                    } else {
                                                        $sgrade = round($sgrade, 2);
                                                        $grade = $sgrade . "/" . $mgrade;
                                                        $dmark = $gid->timemodified;
                                                        $dmark = strftime("%Y-%m-%d", $dmark);
                                                    }
                                                }
                                            }
                                            $ln = $ln.'<td>'.$grade.'</td><td>'.$dmark.'</td></tr>';
                                        }
                                    }
                                }
                            }
                        }
                    }
                    $base = $base.$ln;
                    $base = $base.'</table>';
                    $messagetext = $data->message.' '.$messagetext.'  '.$base;
                    $messagehtml = format_text($data->messagewithsigandattach, $data->format). ' '.$base;
                }
                $subject = trim($subject);
                $lsub = strlen($subject);
                if ($subject == 'N/A' or $lsub < 5) {
                    $subject = 'Progress Report';
                }
                if ($msg == '1') {
                    $success = email_to_user($everyone[$userid], $user, $subject, $messagetext, $messagehtml);
                } else {
                    $userto = $DB->get_record('user', array('id' => $userid));
                    $message = new \core\message\message();
                    $message->component = 'moodle';
                    $message->name = 'instantmessage';
                    $message->userfrom = $from;
                    $message->userto = $userto;
                    $message->subject = $subject;
                    $message->fullmessage = $messagehtml;
                    $message->fullmessageformat = FORMAT_MARKDOWN;
                    $message->fullmessagehtml = $messagehtml;
                    $message->smallmessage = $messagehtml;
                    $message->notification = '0';
                    $message->replyto = $user->email;
                    $message->courseid = SITEID;
                    $success = message_send($message);

                }
                // Get the additional emails.
                $params = array($userid);
                $custom1 = get_config('block_wcln_progress', 'custom1');
                $custom2 = get_config('block_wcln_progress', 'custom2');
                $custom3 = get_config('block_wcln_progress', 'custom3');
                $cfield = '';
                if (!empty($custom1)) {
                    $cfield = $custom1;
                    if (!empty($custom2)) {
                        $cfield = '"'.$cfield.'","'.$custom2.'"';
                    }
                    if (!empty($custom3)) {
                        $cfield = $cfield.'","'.$custom3.'"';
                    }

                } else {
                    if (!empty($custom2)) {
                        $cfield = '"'.$custom2.'"';
                    }
                    if (!empty($custom3)) {
                        $cfield = '"'.$custom3.'"';
                    }
                }
                $cfield = str_replace('""','"', $cfield);
                $clen = strlen($cfield);
                if ($clen > 5) {
                    $resultfield1 = $DB->get_records_sql("select * from
                                    {user_info_field} where shortname IN (".$cfield.")", $params);
                    $i = 0;
                    foreach ($resultfield1 as $field1) {
                        $fid = $field1->id;
                        $params2 = array($userid, $fid);
                        $resultdata1 = $DB->get_records_sql("select * from {user_info_data}
                                            where userid=? and fieldid = ?", $params2);
                        foreach ($resultdata1 as $data1) {
                            $em = trim($data1->data);
                            $sl = strlen($em);
                            if ($sl > 3) {
                                $fakeuser = $DB->get_record('user', array('id' => $userid), '*', MUST_EXIST);
                                $fakeuser->id = 99999900 + $i;
                                $fakeuser->email = $em;
                                $fakeuser->lastname = 'Contact';
                                $fakeuser->firtname = 'Student';
                                $fakeuser->mailformat = 1;
                                $additionalemailsuccess = email_to_user($fakeuser, $user, $subject, $messagetext, $messagehtml);
                                if (!$additionalemailsuccess) {
                                    $data->failuserids[] = $em;
                                    $warnings[] = get_string("no_email_address", 'block_wcln_progress', $fakeuser->email);
                                }
                            }
                        }
                    }
                }
                if (!$success) {
                    $warnings[] = get_string("no_email", 'block_wcln_progress', $everyone[$userid]);
                    $data->failuserids[] = $userid;
                }
                if ($data->receipt) {
                    $messagetextbase = $messagetextbase.'\n'.$messagetext;
                    $messagehtmlbase = $messagehtmlbase.'<br>'.$messagehtml;
                }
                $messagetext = '';
                $messagehtml = '';
                $messagetext = format_text_email($data->messageWithSigAndAttach, $data->format);
                $messagehtml = format_text($data->messageWithSigAndAttach, $data->format);
            }
        }
    }
    $email = $data;
}
if (isset($data->receipt)) {
    if ($data->receipt == 1) {
        email_to_user($USER, $user, $subject, $messagetextbase, $messagehtmlbase);
    }
}
$form->set_data($email);
if (empty($warnings)) {
    if (isset($email->send)) {
        redirect(new moodle_url('/blocks/wcln_progress/emaillog.php', array('courseid' => $course->id)));
    } else if (isset($email->draft)) {
        $warnings['success'] = get_string("changessaved");
    }
}
echo $OUTPUT->header();
echo $OUTPUT->heading($blockname);

foreach ($warnings as $type => $warning) {
    $class = ($type === 'success') ? 'notifysuccess' : 'notifyproblem';
    echo $OUTPUT->notification($warning, $class);
}

echo html_writer::start_tag('div', array('class' => 'no-overflow'));
$form->display();

echo html_writer::end_tag('div');
echo $OUTPUT->footer();
