<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/*
 * View log for wcln_progress.
 *
 * @package   block_wcln_progress
 * @copyright 2020 WCLN
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once('../../config.php');
require_once('lib.php');
require_login();
$courseid = required_param('courseid', PARAM_INT);
$type = optional_param('type', 'log', PARAM_ALPHA);
$typeid = optional_param('typeid', 0, PARAM_INT);
$action = optional_param('action', null, PARAM_ALPHA);
$page = optional_param('page', 0, PARAM_INT);
$perpage = optional_param('perpage', 10, PARAM_INT);
$userid = optional_param('userid', $USER->id, PARAM_INT);

if (!$course = $DB->get_record('course', array('id' => $courseid))) {
    print_error('no_course', 'block_wcln_progress', '', $courseid);
}

$context = context_course::instance($courseid);

// Has to be in on of these.
if (!in_array($type, array('log', 'drafts'))) {
    print_error('not_valid', 'block_wcln_progress', '', $type);
}

$canimpersonate = has_capability('block/wcln_progress:canimpersonate', $context);
if (!$canimpersonate and $userid != $USER->id) {
    print_error('not_valid_user', 'block_wcln_progress');
}

$validactions = array('delete', 'confirm');

$cansend = has_capability('block/wcln_progress:cansend', $context);

$properpermission = ($cansend or !empty($config['allowstudents']));

// Managers can delete by capability 'candelete'.
// Those with 'cansend' (incl students, if $config['allowstudents']) can only delete drafts.
$candelete = (has_capability('block/wcln_progress:candelete', $context) or ($cansend and $type == 'drafts')
        or ($properpermission and $type == 'drafts'));

// Stops students from tempering with history.
if (!$properpermission or (!$candelete and in_array($action, $validactions))) {
    print_error('no_permission', 'block_wcln_progress');
}

if (isset($action) and !in_array($action, $validactions)) {
    print_error('not_valid_action', 'block_wcln_progress', '', $action);
}

if (isset($action) and empty($typeid)) {
    print_error('not_valid_typeid', 'block_wcln_progress', '', $action);
}

$blockname = wcln_progress::_s('pluginname');
$header = wcln_progress::_s($type);

$PAGE->set_context($context);
$PAGE->set_course($course);
$PAGE->navbar->add($blockname);
$PAGE->navbar->add($header);
$PAGE->set_title($blockname . ': ' . $header);
$PAGE->set_heading($blockname . ': ' . $header);
$PAGE->set_url('/blocks/wcln_progress/emaillog.php', array('courseid' => $courseid));
$PAGE->set_pagetype(wcln_progress::PAGE_TYPE);

$dbtable = 'block_wcln_progress_' . $type;

$params = array('userid' => $userid, 'courseid' => $courseid);
$count = $DB->count_records($dbtable, $params);

switch ($action) {
    case "confirm":
        if (wcln_progress::cleanup($dbtable, $context->id, $typeid)) {
            $url = new moodle_url('/blocks/wcln_progress/emaillog.php', array(
                'courseid' => $courseid,
                'type' => $type,
                'groups' => 0
            ));
            redirect($url);
        } else {
            print_error('delete_failed', 'block_wcln_progress', '', $typeid);
        }
    case "delete":
        $html = wcln_progress::delete_dialog($courseid, $type, $typeid);
        break;
    default:
        $html = wcln_progress::list_entries($courseid, $type, $page, $perpage, $userid, $count, $candelete);
}

if ($courseid == SITEID) {
    $html .= html_writer::link(
        new moodle_url('/blocks/wcln_progress/admin_email.php'),
        wcln_progress::_s('composenew')
     );
} else {
    $html .= html_writer::link(
        new moodle_url(
            '/blocks/wcln_progress/email.php',
            array('courseid' => $courseid,
                  'groups' => 0)),
        wcln_progress::_s('composenew')
    );
}

if ($canimpersonate and $USER->id != $userid) {
    $user = $DB->get_record('user', array('id' => $userid));
    $header .= ' for '. fullname($user);
}
echo $OUTPUT->header();
echo $OUTPUT->heading($header);
if ($canimpersonate) {
    $getnamestring = 'u.firstname, u.lastname';
    if ($CFG->version >= 2013111800) {
        $getnamestring = get_all_user_name_fields(true, 'u');
    }
    $sql = "SELECT DISTINCT(l.userid)," . $getnamestring . "
                FROM {block_wcln_progress_$type} l,
                     {user} u
                WHERE u.id = l.userid AND courseid = ? ORDER BY u.lastname";
    $users = $DB->get_records_sql($sql, array($courseid));
    $useroptions = array_map(function($user) {
        return fullname($user);
    }, $users);
    $url = new moodle_url('emaillog.php', array(
        'courseid' => $courseid,
        'type' => $type,
        'groups' => '0'
    ));

    $defaultoption = array('' => wcln_progress::_s('select_users'));
    echo $OUTPUT->single_select($url, 'userid', $useroptions, $userid, $defaultoption);
}
if (empty($count)) {
    echo $OUTPUT->notification(wcln_progress::_s('no_'.$type));
    if ($COURSE->id == 1) {
        echo $OUTPUT->continue_button('/blocks/wcln_progress/admin_email.php?courseid='.$courseid.'&groups=0');
    } else {
        echo $OUTPUT->continue_button('/blocks/wcln_progress/email.php?courseid='.$courseid.'&groups=0');
    }

    echo $OUTPUT->footer();
    exit;
}
echo $html;
echo $OUTPUT->footer();
